package core.export;

import core.structures.ITask;
import core.structures.TodoList;

/**
 * The Text Export class. Part of the Template Method Pattern.
 *
 * This class exports the given TodoList to a fancy html file.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public class HTMLExport extends AbstractExport {

	private String	nl	= System.getProperty("line.separator"); // the system's new line character

	@Override
	synchronized String prepareString(TodoList tdl) {
		StringBuilder sb = new StringBuilder();
		sb.append("<html>" + nl);
		sb.append("<head>" + nl);
		sb.append(" <title>TodoList: " + tdl.getProjectName() + "</title>" + nl);
		sb.append(" <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">" + nl);
		sb.append(" <style type=\"text/css\">" + nl);
		sb.append(" BODY			{" + nl);
		sb.append("	font-size: 10pt;" + nl);
		sb.append("	font-family: Arial, Tahoma, Verdana;" + nl);
		sb.append("}" + nl);
		sb.append("H1				{" + nl);
		sb.append("	font-family: Tahoma, Arial, Sans-Serif;" + nl);
		sb.append("	font-size: 22pt;" + nl);
		sb.append("}" + nl);
		sb.append("H2				{" + nl);
		sb.append("	font-family: Tahoma, Arial, Sans-Serif;" + nl);
		sb.append("	font-size: 16pt;" + nl);
		sb.append("}" + nl);
		sb.append("H3				{" + nl);
		sb.append("	font-family: Tahoma, Arial, Sans-Serif;" + nl);
		sb.append("}" + nl);
		sb.append("H3 > B			{" + nl);
		sb.append("	font-family: Times-New-Roman;" + nl);
		sb.append("	font-weight: bold;" + nl);
		sb.append("}" + nl);
		sb.append("TT					{" + nl);
		sb.append("	font-weight: bold;" + nl);
		sb.append("}" + nl);
		sb.append("DIV.task > table {" + nl);
		sb.append("  border: 1px dashed silver;" + nl);
		sb.append("}" + nl);
		sb.append(" </style>" + nl);
		sb.append(" <script language=\"javascript\">" + nl);
		sb.append("   function show_div(id) {" + nl);
		sb.append("     document.getElementById(id).style.display = 'inline';" + nl);
		sb.append("   }" + nl);
		sb.append("   function hide_div(id) {" + nl);
		sb.append("     document.getElementById(id).style.display = 'none';" + nl);
		sb.append("   }" + nl);
		sb.append(" </script>" + nl);
		sb.append("</head>" + nl);
		sb.append("" + nl);
		sb.append("<body>" + nl);
		sb.append("<p>" + nl);
		sb.append("<h1>TodoList</h1>" + nl);
		sb.append("<ul id=\"todolist_" + tdl.getProjectName() + "\" class=\"todolist\">" + nl);
		for (int i = 0; i < tdl.numTasks(); i++) {
			sb.append(prepareString(tdl.getTask(i)));
		}
		sb.append("</ul>" + nl);
		sb.append("</p>" + nl);
		sb.append("</body>" + nl);
		sb.append("</html>" + nl);
		return sb.toString();
	}

	private String prepareString(ITask t) {
		StringBuilder sb = new StringBuilder();
		sb.append("  <!-- begin task -->" + nl);
		sb.append("  <li>" + nl);
		sb.append("  <b>" + t.getTitle() + "</b>" + nl);
		sb.append("  [ <a href=\"javascript:show_div('details_" + t.getTitle() + "');\">more &raquo;</a> ]<br />" + nl);
		sb.append("  <div id=\"details_" + t.getTitle() + "\" class=\"task\" style=\"display:none;\">" + nl);
		sb.append("    <table border=\"0\">" + nl);
		sb.append("         <tr>" + nl);
		sb.append("     <td>" + nl);
		sb.append("       [ <a href=\"javascript:hide_div('details_" + t.getTitle() + "');\">&laquo; less</a> ]" + nl);
		sb.append("     </td>" + nl);
		sb.append("    </tr>" + nl);
		sb.append("    <tr>" + nl);
		sb.append("     <td><b>Title</b>:</td><td>" + t.getTitle() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Priority:</td><td>" + t.getPriority() + "</td>" + nl);
		sb.append("     <td>Risk:</td><td>" + t.getRisk() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("      <td>Percent done:</td><td>" + t.getPercentdone() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Start Date:</td><td>" + t.getStartDate() + "</td>" + nl);
		sb.append("     <td>Done Date:</td><td>" + t.getDoneDate() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Last Modification:</td><td>" + t.getLastModifiedDate() + "</td>" + nl);
		sb.append("     <td>Due Date:</td><td>" + t.getDueDate() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Status:</td><td>" + t.getStatus() + "</td>" + nl);
		sb.append("     <td>Allocated by:</td><td>" + t.getAllocatedBy() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Person:</td><td>" + t.getPerson() + "</td>" + nl);
		sb.append("     <td>Depends:</td><td>" + t.getDepends() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Creation Date:</td><td>" + t.getCreationDate() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Time spent:</td><td>" + t.getTimeSpent() + " " + t.getTimeSpentUnits() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>External ID:</td><td>" + t.getExternalId() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Time Estimated:</td><td>" + t.getTimeEstimated() + " " + t.getTimeEstimatedUnits()
				+ "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Category:</td><td>" + t.getCategory() + "</td>" + nl);
		sb.append("     <td>Cost:</td><td>" + t.getCost() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("     <tr>" + nl);
		sb.append("     <td>Comments:</td><td>" + t.getComments() + "</td>" + nl);
		sb.append("     </tr>" + nl);
		sb.append("    </table>" + nl);
		sb.append("  </div>" + nl);
		if (t.numTasks() > 0) {
			sb.append("  <ul id=\"task_TITLE\" class=\"task\">" + nl);
			for (int i = 0; i < t.numTasks(); i++) {
				sb.append(prepareString(t.getTask(i)));
			}
			sb.append("  </ul>" + nl);
		}
		sb.append("  </li>" + nl);
		sb.append("  <!-- end task -->" + nl);
		return sb.toString();
	}

}
