package core.export;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import core.structures.ITask;
import core.structures.TodoList;
import core.utils.XMLTools;

/**
 * The Text Export class. Part of the Template Method Pattern.
 *
 * This class exports the given TodoList to a plain textfile.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public class TextExport extends AbstractExport {

	private String	nl	= System.getProperty("line.separator"); // the system's new line character

	@Override
	synchronized String prepareString(TodoList tdl) {
		StringBuilder sb = new StringBuilder();
		sb.append("Project Name: " + tdl.getProjectName() + nl);
		sb.append("Last Modified: " + tdl.getLastModified() + nl);
		sb.append("Fileformat: " + tdl.getFileformat() + nl);
		sb.append("Fileversion: " + tdl.getFileversion() + nl);
		sb.append("Next Unique ID: " + tdl.getNextuniqeid() + nl);
		sb.append("Custom Comments Type: " + tdl.getCustomCommentsType() + nl);
		for (int i = 0; i < tdl.numTasks(); i++) {
			sb.append(prepareString(tdl.getTask(i), 1) + nl);
		}
		return sb.toString();
	}

	// FIXME unfertig
	private String prepareString(ITask t, int indent) {
		StringBuilder sb = new StringBuilder();
		sb.append(XMLTools.getNumChars(' ', indent) + "DONE: [     ]" + XMLTools.getNumChars('-', 65) + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Title: " + t.getTitle() + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Priority: " + processBar(22, t.getPriority(), 100) + " ");
		sb.append(XMLTools.getNumChars(' ', indent) + "XXXRisk: " + processBar(22, t.getRisk(), 100) + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Percent done: " + processBar(42, t.getPercentdone(), 100) + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + formatNamedTextField("Due Date: ", t.getDueDate().toString(), 60)
				+ nl);
		sb.append(XMLTools.getNumChars(' ', indent)
				+ formatForMaxWidth(formatDateField("Start Date", t.getStartDate()), formatDateField("Done Date", t
						.getDoneDate()), 78) + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + formatDateField("Last Modification: ", t.getLastModifiedDate())
				+ " " + formatDateField("Creation Date: ", t.getCreationDate()) + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Status: " + t.getStatus() + " ");
		sb.append(XMLTools.getNumChars(' ', indent) + "Allocated by: " + t.getAllocatedBy() + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Person: " + t.getPerson() + " ");
		sb.append(XMLTools.getNumChars(' ', indent) + "Depends: " + t.getDepends() + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "External ID: " + t.getExternalId() + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Category: " + t.getCategory() + " ");
		sb.append(XMLTools.getNumChars(' ', indent) + "Cost: " + t.getCost() + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + "Time spent: " + t.getTimeSpent() + " [" + t.getTimeSpentUnits()
				+ "] ");
		sb.append(XMLTools.getNumChars(' ', indent) + "Time Estimated: " + t.getTimeEstimated() + " ["
				+ t.getTimeEstimatedUnits() + "]" + nl);
		sb.append(XMLTools.getNumChars(' ', indent)
				+ "Comments: -------------------------------------------------------------------" + nl);
		sb.append(XMLTools.getNumChars(' ', indent) + t.getComments() + nl);
		sb.append(XMLTools.getNumChars(' ', indent)
				+ "-----------------------------------------------------------------------------");
		for (int i = 0; i < t.numTasks(); i++) {
			sb.append(prepareString(t.getTask(i), indent + 1) + nl);
		}
		sb.setLength(sb.length() - nl.length());// remove the last line-feed
		return sb.toString();
	}

	/**
	 * creates a string based process bar thats excatly or less than
	 * width wide, represents the value value and is capable of displaying
	 * a max. value of max.
	 * @param width
	 * @param value
	 * @param max
	 * @return
	 * FIXME Mit StringBuilder machen
	 */
	private static String processBar(int width, int value, int max) {
		String str = "[";
		String str2 = " (" + value + "/" + max + ")";
		int avail = width - (2 + str2.length()); // for the brackets ([and ]) and the summary
		int blocks_filled = Math.round((((value * 100) / max) * avail) / 100);
		//System.out.println("avail: " + avail + ", max: " + max + ", value: " + value + ", bf: " + blocks_filled);
		for (int i = 0; i < blocks_filled; i++) {
			str += "#";
		}
		for (int i = str.length(); i < avail + 1; i++) {
			str += "-"; // an empty block
		}
		str += "]" + str2;
		return str;
	}

	/**
	 * creates a String that consists of the String left and the String left,
	 * seperated by spaces, but not exceeding a width of maxWidth.
	 * @param left
	 * @param right
	 * @param max
	 * @return
	 */
	private static String formatForMaxWidth(String left, String right, int maxWidth) {
		if (left.length() + right.length() >= maxWidth) {
			return left + " " + right;
		} else {
			int spacing = maxWidth - (left.length() + right.length());
			return left + XMLTools.getNumChars(' ', spacing) + right;
		}
	}

	private static String formatDateField(String label, Date value) {
		String str = label + ": [";
		Calendar cal = new GregorianCalendar();
		cal.setTime(value);
		str += cal.get(Calendar.DAY_OF_MONTH) + ".";
		str += cal.get(Calendar.MONTH) + ".";
		str += cal.get(Calendar.YEAR);
		str += "]";
		return str;
	}

	/**
	 * formats a string that is no more than maxWidth chars wide,
	 * has the label label and the value value.
	 * Example: "Name: [   John Doe]" <-> formatNamedTextField("Name", "John Doe", 19);
	 * @param label
	 * @param value
	 * @param maxWidth
	 * @return
	 */
	private static String formatNamedTextField(String label, String value, int maxWidth) {
		String str = label;
		str += ": ";
		str += "[";
		int spacing = maxWidth - (str.length() + 1 + value.length()); // + 1 for the last bracket ]
		str += XMLTools.getNumChars(' ', spacing);
		str += value;
		str += "]";
		return str;
	}

}
