package core.export;

import org.jdom.Element;
import core.fileformat.TodoListExe;
import core.structures.ITask;
import core.structures.TodoList;
import core.utils.XMLTools;

/**
 * The XML Export class. Part of the Template Method Pattern.
 *
 * This class exports the given TodoList to a TodoList.exe compatible XML file.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public class XMLExport extends AbstractExport {

	@Override
	protected String prepareString(TodoList tdl) {
		Element root = new Element("TODOLIST");
		root.setAttribute("FIELFORMAT", new Integer(tdl.getFileformat()).toString());
		root.setAttribute("FILEVERSION", new Integer(tdl.getFileversion()).toString());
		root.setAttribute("NEXTUNIQUEID", new Integer(tdl.getNextuniqeid()).toString());
		if (tdl.getLastModified() != null)
			root.setAttribute("LASTMODIFIED", tdl.getLastModified().toString());
		else
			root.setAttribute("LASTMODIFIED", "");
		root.setAttribute("CUSTOMCOMMENTSTYPE", tdl.getCustomCommentsType());
		for (int i = 0; i < tdl.numTasks(); i++) {
			root.addContent(prepareXML(tdl.getTask(i)));
		}
		return XMLTools.xmlToString(root);
	}

	private Element prepareXML(ITask t) {
		Element node = new Element(TodoListExe.TASK);
		node.setAttribute(TodoListExe.PRIORITYCOLOR, new Integer(t.getPriorityColor()).toString());
		if (t.getStartDate() != null)
			node.setAttribute(TodoListExe.STARTDATESTRING, t.getStartDate().toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.RISK, new Integer(t.getRisk()).toString());
		if (t.getDoneDate() != null)
			node.setAttribute(TodoListExe.LASTMODSTRING, t.getDoneDate().toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.PRIORITY, new Integer(t.getPriority()).toString());
		if (t.getStatus() != null)
			node.setAttribute(TodoListExe.STATUS, t.getStatus());
		if (t.getDueDate() != null)
			node.setAttribute(TodoListExe.DUEDATESTRING, t.getDueDate().toString());
		// TODO Calcpercentdone
		if (t.getAllocatedBy() != null)
			node.setAttribute(TodoListExe.ALLOCATEDBY, t.getAllocatedBy());
		if (t.getLastModifiedDate() != null)
			node.setAttribute(TodoListExe.LASTMOD, t.getLastModifiedDate().toString()); // TODO Date konvertieren
		if (t.getCreationDate() != null)
			node.setAttribute(TodoListExe.CREATIONDATESTRING, t.getCreationDate().toString()); // TODO Date konvertieren
		if (t.getTitle() != null)
			node.setAttribute(TodoListExe.TITLE, t.getTitle());
		if (t.getCreationDate() != null)
			node.setAttribute(TodoListExe.CREATIONDATE, t.getCreationDate().toString()); // TODO Date konvertieren
		if (t.getTimeSpentUnits() != null)
			node.setAttribute(TodoListExe.TIMESPENTUNITS, t.getTimeSpentUnits());
		if (t.getPerson() != null)
			node.setAttribute(TodoListExe.PERSON, t.getPerson());
		node.setAttribute(TodoListExe.TIMEESTIMATE, new Double(t.getTimeEstimated()).toString());
		// TODO Calctimespent
		// TODO numcategory
		if (t.getCategory() != null)
			node.setAttribute(TodoListExe.CATEGORY, t.getCategory());
		if (t.getComments() != null)
			node.setAttribute(TodoListExe.COMMENTS, t.getComments());
		node.setAttribute(TodoListExe.PRIORITYWEBCOLOR, t.getPriorityWebColor());
		node.setAttribute(TodoListExe.TIMESPENT, new Double(t.getTimeSpent()).toString());
		if (t.getFileReferingPath() != null)
			node.setAttribute(TodoListExe.FILEREFPATH, t.getFileReferingPath());
		node.setAttribute(TodoListExe.ID, new Integer(t.getId()).toString());
		if (t.getDueDate() != null)
			node.setAttribute(TodoListExe.DUEDATE, t.getDueDate().toString());
		node.setAttribute(TodoListExe.COST, new Double(t.getCost()).toString());
		// TODO customcomments
		// TODO calctimestimate
		// TODO calccost
		node.setAttribute(TodoListExe.PERCENTDONE, new Integer(t.getPercentdone()).toString());
		if (t.getDepends() != null)
			node.setAttribute(TodoListExe.DEPENDS, t.getDepends());
		if (t.getStartDate() != null)
			node.setAttribute(TodoListExe.STARTDATE, t.getStartDate().toString()); // TODO Date konvertieren
		if (t.getDoneDate() != null)
			node.setAttribute(TodoListExe.DONEDATE, t.getDoneDate().toString()); // TODO Date konvertieren
		if (t.getDoneDate() != null)
			node.setAttribute(TodoListExe.DONEDATESTRING, t.getDoneDate().toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.POS, new Integer(t.getPos()).toString());
		for (int i = 0; i < t.numTasks(); i++) {
			node.addContent(prepareXML(t.getTask(i)));
		}
		return node;
	}

}
