package core.gui;

import java.io.File;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import core.export.HTMLExport;
import core.export.IExport;
import core.export.TextExport;
import core.structures.ITask;
import core.structures.Task;
import core.structures.TodoList;
import core.utils.XMLTools;

/**
 *  A preliminary Command-Line-Interface (CLI).
 *
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public class CommandLineInterface {

	static Logger	logger	= Logger.getLogger(CommandLineInterface.class);

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		BasicConfigurator.configure();
		logger.setLevel(Level.TRACE);
		TodoList tdl1 = new TodoList("TDL1");
		TodoList tdl2 = new TodoList("TDL1");
		ITask task1 = new Task("Task1");
		ITask task2 = new Task("Task2");
		task2.setPercentdone(50);
		task2.setRisk(34);
		task2.setPriority(99);
		task1.addTask(task2);
		task1.addTask(task2);
		task1.addTask(task2);
		task1.addTask(task2);
		tdl1.addTask(task1);
		tdl1.addTask(task2);
		tdl2.addTask(task1);
		tdl2.addTask(task2);
		logger.info("TDL1: " + tdl1);
		logger.info("TDL2: " + tdl2);
		logger.info("Equals? " + tdl1.equals(tdl2));
		IExport export = null;
		File outputFile = new File(XMLTools.getTempFilename());
		export = new HTMLExport();
		export = new TextExport();
		export.setOutputFile(outputFile);
		export.setTodoList(tdl1);
		export.export();
		/*

		 int runs = 1500;
		 long startTimeText = System.currentTimeMillis();
		 for (int i = 0; i < runs; i++) {
		 export = new TextExport();
		 export.setOutputFile(outputFile);
		 export.setTodoList(tdl1);
		 export.export();
		 export = null;
		 outputFile.delete();
		 }
		 long endTimeText = System.currentTimeMillis();
		 long startTimeCsv = System.currentTimeMillis();
		 for (int i = 0; i < runs; i++) {
		 export = new CSVExport();
		 export.setOutputFile(outputFile);
		 export.setTodoList(tdl1);
		 export.export();
		 export = null;
		 outputFile.delete();
		 }
		 long endTimeCsv = System.currentTimeMillis();
		 long runtimeCsv = endTimeCsv - startTimeCsv;
		 long runtimeText = endTimeText - startTimeText;
		 logger.info("Runtime CSV(Sb as argument): " + runtimeCsv);
		 logger.info("Runtime Text(toString as return type): " + runtimeText);
		 */
	}

}
