package core.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import com.toedter.calendar.JDateChooser;

import core.export.HTMLExport;
import core.export.IExport;
import core.export.TextExport;
import core.export.XMLExport;
import core.gui.views.TreemapPriorityView;
import core.gui.views.TreemapRiskView;
import core.structures.ITask;
import core.structures.Task;
import core.structures.TodoList;
import core.utils.XMLTools;
import core.utils.comparators.DueDateComparator;
import core.utils.comparators.PriorityComparator;
import core.utils.comparators.RiskComparator;

/**
 *  The GUI.
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  The getters for the private gui fields are similar to the proxy pattern.
 *
 *  @author tex
 */
public class Gui {

	static Logger	logger	= Logger.getLogger(Gui.class);	// @jve:decl-index=0:

	/**
	 * Launches this application
	 *
	 * @throws IOException
	 * @throws JDOMException
	 */
	public static void main(String[] args) throws JDOMException, IOException {
		BasicConfigurator.configure();
		logger.setLevel(Level.ERROR);
		final String filename;
		if (args.length > 0) {
			filename = args[0];
		} else {
			filename = "";
		}
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				TodoList tdl = new TodoList("ToDoList");
				Gui application = new Gui(tdl);
				if (filename.length() > 0) {
					application.readFromFile(new File(filename));
				}
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
					SwingUtilities.updateComponentTreeUI(application.getJFrame());
				} catch (ClassNotFoundException e) {
					logger.error("ClassNotFoundExceptin", e);
				} catch (InstantiationException e) {
					logger.error("InstantiationException", e);
				} catch (IllegalAccessException e) {
					logger.error("IllegalAccessException", e);
				} catch (UnsupportedLookAndFeelException e) {
					logger.error("UnsupportedLookAndFeelException", e);
				}
				application.getJFrame().setVisible(true);
			}
		});
	}

	private JFrame				jFrame						= null; // @jve:decl-index=0:visual-constraint="10,10"

	private JPanel				jContentPane				= null;

	private JPanel				jTaskDetailPane				= null;

	private ITask				currentTask					= null; // @jve:decl-index=0:

	private JMenuBar			jJMenuBar					= null;

	private JMenu				fileMenu					= null;

	private JMenu				editMenu					= null;

	private JMenu				helpMenu					= null;

	private JMenuItem			exitMenuItem				= null;

	private JMenuItem			aboutMenuItem				= null;

	private JMenuItem			saveMenuItem				= null;

	private JDialog				aboutDialog					= null; // @jve:decl-index=0:visual-constraint="725,18"

	private JPanel				aboutContentPane			= null;

	private JLabel				aboutVersionLabel			= null;

	private JTree				jTree						= null;

	private JTextPane			jTextPane					= null;

	private TodoList			tdl							= null;

	private JScrollPane			jScrollPane					= null;

	private JSplitPane			jSplitPane					= null;

	private GridBagLayout		gridBagLayout				= null;

	private GridBagConstraints	gridBagConstraints			= null;

	private JLabel				titleLabel					= null;

	private JTextField			titleTextField				= null;

	private JLabel				priorityLabel				= null;

	private JSlider				prioritySlider				= null;

	private JLabel				percentDoneLabel			= null;

	private JSlider				percentDoneSlider			= null;

	private JLabel				riskLabel					= null;

	private JSlider				riskSlider					= null;

	private JLabel				startDateLabel				= null;

	private JDateChooser		startDateChooser			= null;

	private JLabel				doneDateLabel				= null;

	private JDateChooser		doneDateChooser				= null;

	private JLabel				lastModLabel				= null;

	private JLabel				lastModDateLabel			= null;

	private JLabel				dueDateLabel				= null;

	private JDateChooser		dueDateChooser				= null;

	private JLabel				statusLabel					= null;

	private JTextField			statusTextField				= null;

	private JLabel				allocatedByLabel			= null;

	private JTextField			allocatedByTextField		= null;

	private JLabel				personLabel					= null;

	private JTextField			personTextField				= null;

	private JLabel				dependsLabel				= null;

	private JTextField			dependsTextField			= null;

	private JLabel				creationDateLabel			= null;

	private JDateChooser		creationDateChooser			= null;

	private JLabel				timeSpentLabel				= null;

	private JTextField			timeSpentTextField			= null;

	private JLabel				timeSpentUnitsLabel			= null;

	private JTextField			timeSpentUnitsTextField		= null;

	private JLabel				externalIdLabel				= null;

	private JTextField			externalIdTextField			= null;

	private JLabel				timeEstLabel				= null;

	private JTextField			timeEstTextField			= null;

	private JLabel				timeEstUnitsLabel			= null;

	private JTextField			timeEstUnitsTextField		= null;

	private JLabel				categoryLabel				= null;

	private JTextField			categoryTextField			= null;

	private JLabel				costLabel					= null;

	private JTextField			costTextField				= null;

	private JLabel				fileRefPathLabel			= null;

	private JTextField			fileRefPathTextField		= null;

	private JLabel				commentsLabel				= null;

	private JScrollPane			commentsTextAreaScrollPane	= null;

	private JTextArea			commentsTextArea			= null;

	private JMenu				lookAndFeelMenu				= null;

	private JMenuItem			systemLookAndFeelItem		= null;

	private JMenuItem			defaultLookAndFeelItem		= null;

	private JMenuItem			motifLookAndFeelItem		= null;

	private JMenuItem			plasticLookAndFeelItem		= null;

	private JMenu				importSubMenu				= null;

	private JMenuItem			importFromXMLMenuItem		= null;

	private JMenu				exportSubMenu				= null;

	private JMenuItem			exportAsXMLMenuItem			= null;

	private JMenuItem			exportAsCSVMenuItem			= null;

	private JMenuItem			exportAsHTMLMenuItem		= null;

	private JMenuItem			exportAsTextMenuItem		= null;

	private JMenuItem			exportAsPDFMenuItem			= null;

	private JMenuItem			exportAsLatexMenuItem		= null;

	private JMenuItem			closeMenuItem				= null;

	private JMenuItem			openMenuItem				= null;

	private JMenuItem			saveToMenuItem				= null;

	private File				openSaveFile				= null;

	private JMenuItem			newMenuItem					= null;

	private JMenu				viewMenu					= null;

	private JMenuItem			asTreeItem					= null;

	private JMenuItem			asListByDueDateItem			= null;

	private JMenuItem			asListByPriorityItem		= null;

	private JMenuItem			asListByRiskItem			= null;

	private JMenuItem			removeSubnodeItem			= null;

	private JMenuItem			addSubnodeItem				= null;

	private ITask[]				currentTaskList				= null;

	private JList				jList						= null;

	private JLabel				aboutProductNameLabel		= null;

	private JLabel				aboutCopyrightLabel			= null;

	private JLabel				aboutUrlLabel				= null;

	private JLabel				aboutGplLabel				= null;

	public Gui(TodoList tdl) {
		this.tdl = tdl;
	}

	private void closeActiveList() {
		this.tdl = null;
		this.tdl = new TodoList("ToDoList");
		this.refreshJTree();
	}

	protected void displayTask() {
		this.getJTextPane().setText("Please select a task");
	}

	protected void displayTask(ITask t) {
		int dividerSize = this.getJSplitPane().getDividerLocation();
		this.getJTextPane().setText(t.prettyFormat(0));
		this.jSplitPane.remove(this.jTaskDetailPane);
		this.getJSplitPane().setRightComponent(this.getTaskDetailsPane(t));
		this.getJSplitPane().setDividerLocation(dividerSize);
	}

	/**
	 * This method initializes aboutContentPane
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getAboutContentPane() {
		if (aboutContentPane == null) {
			aboutContentPane = new JPanel();
			aboutContentPane.setLayout(new BorderLayout());
			aboutContentPane.add(getAboutGplLabel(), BorderLayout.NORTH);
			aboutContentPane.add(getAboutProductNameLabel(), BorderLayout.WEST);
			aboutContentPane.add(getAboutVersionLabel(), BorderLayout.CENTER);
			aboutContentPane.add(getAboutCopyrightLabel(), BorderLayout.EAST);
			aboutContentPane.add(getAboutUrlLabel(), BorderLayout.SOUTH);
		}
		return aboutContentPane;
	}

	private JLabel getAboutCopyrightLabel() {
		if (this.aboutCopyrightLabel == null) {
			this.aboutCopyrightLabel = new JLabel();
			this.aboutCopyrightLabel.setText("(c) 2007 Dominik Schulz <ds@gauner.org>");
		}
		return this.aboutCopyrightLabel;
	}

	/**
	 * This method initializes aboutDialog
	 *
	 * @return javax.swing.JDialog
	 */
	private JDialog getAboutDialog() {
		if (aboutDialog == null) {
			aboutDialog = new JDialog(getJFrame(), true);
			aboutDialog.setTitle("About");
			aboutDialog.setContentPane(getAboutContentPane());
		}
		return aboutDialog;
	}

	private JLabel getAboutGplLabel() {
		if (this.aboutGplLabel == null) {
			this.aboutGplLabel = new JLabel();
			this.aboutGplLabel.setText("This Progam is released under the terms of the GPL.");
		}
		return this.aboutGplLabel;
	}

	/**
	 * This method initializes jMenuItem
	 *
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getAboutMenuItem() {
		if (aboutMenuItem == null) {
			aboutMenuItem = new JMenuItem();
			aboutMenuItem.setText("About");
			aboutMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					JDialog aboutDialog = getAboutDialog();
					aboutDialog.pack();
					Point loc = getJFrame().getLocation();
					loc.translate(20, 20);
					aboutDialog.setLocation(loc);
					aboutDialog.setVisible(true);
				}
			});
		}
		return aboutMenuItem;
	}

	private JLabel getAboutProductNameLabel() {
		if (this.aboutProductNameLabel == null) {
			this.aboutProductNameLabel = new JLabel();
			this.aboutProductNameLabel.setText("jTodoList");
		}
		return this.aboutProductNameLabel;
	}

	private JLabel getAboutUrlLabel() {
		if (this.aboutUrlLabel == null) {
			this.aboutUrlLabel = new JLabel();
			this.aboutUrlLabel.setText("http://developer.gauner.org/jtodolist/");
		}
		return this.aboutUrlLabel;
	}

	/**
	 * This method initializes aboutVersionLabel
	 *
	 * @return javax.swing.JLabel
	 */
	private JLabel getAboutVersionLabel() {
		if (aboutVersionLabel == null) {
			aboutVersionLabel = new JLabel();
			aboutVersionLabel.setText("Version 0.3");
			aboutVersionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return aboutVersionLabel;
	}

	private JMenuItem getAddSubnodeItem() {
		if (addSubnodeItem == null) {
			addSubnodeItem = new JMenuItem();
			addSubnodeItem.setText("Add Subnode");
			addSubnodeItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, Event.CTRL_MASK, true));
			addSubnodeItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.addTask(new Task("a Subtask"));
					Gui.this.refreshJTree();
					Gui.this.getJFrame().validate();
					logger.info("Should have added new Subtask");
				}
			});
		}
		return addSubnodeItem;
	}

	private JLabel getAllocatedByLabel() {
		if (this.allocatedByLabel == null) {
			this.allocatedByLabel = new JLabel();
			this.allocatedByLabel.setText("Allocated by: ");
		}
		return this.allocatedByLabel;
	}

	private JTextField getAllocatedByTextField() {
		if (this.allocatedByTextField == null) {
			this.allocatedByTextField = new JTextField();
			this.allocatedByTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setAllocatedBy(((JTextField) e.getSource()).getText());
					logger.info("Set Allocated by to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.allocatedByTextField;
	}

	private JMenuItem getAsListByDueDateItem() {
		if (asListByDueDateItem == null) {
			asListByDueDateItem = new JMenuItem();
			asListByDueDateItem.setText("List (by Due Date)");
			asListByDueDateItem.addActionListener(new ActionListener() {
				@SuppressWarnings("unchecked")
				public void actionPerformed(ActionEvent e) {
					Gui.this.showJList(new DueDateComparator());
				}
			});
		}
		return asListByDueDateItem;
	}

	private JMenuItem getAsListByPriorityItem() {
		if (asListByPriorityItem == null) {
			asListByPriorityItem = new JMenuItem();
			asListByPriorityItem.setText("List (by Priority)");
			asListByPriorityItem.addActionListener(new ActionListener() {
				@SuppressWarnings("unchecked")
				public void actionPerformed(ActionEvent e) {
					Gui.this.showJList(new PriorityComparator());
				}
			});
		}
		return asListByPriorityItem;
	}

	private JMenuItem getAsListByRiskItem() {
		if (asListByRiskItem == null) {
			asListByRiskItem = new JMenuItem();
			asListByRiskItem.setText("List (by Risk)");
			asListByRiskItem.addActionListener(new ActionListener() {
				@SuppressWarnings("unchecked")
				public void actionPerformed(ActionEvent e) {
					Gui.this.showJList(new RiskComparator());
				}
			});
		}
		return asListByRiskItem;
	}

	private JMenuItem getAsTreeItem() {
		if (asTreeItem == null) {
			asTreeItem = new JMenuItem();
			asTreeItem.setText("Tree");
			asTreeItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.refreshJTree();
				}
			});
		}
		return asTreeItem;
	}

	private JLabel getCategoryLabel() {
		if (this.categoryLabel == null) {
			this.categoryLabel = new JLabel();
			this.categoryLabel.setText("Category: ");
		}
		return this.categoryLabel;
	}

	private JTextField getCategoryTextField() {
		if (this.categoryTextField == null) {
			this.categoryTextField = new JTextField();
			this.categoryTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setCategory(((JTextField) e.getSource()).getText());
					logger.info("Set category to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.categoryTextField;
	}

	private JMenuItem getCloseMenuItem() {
		if (this.closeMenuItem == null) {
			this.closeMenuItem = new JMenuItem();
			this.closeMenuItem.setText("Close");
			this.closeMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK, true));
			this.closeMenuItem.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					Gui.this.closeActiveList();
				}

			});
		}
		return this.closeMenuItem;
	}

	private JLabel getCommentsLabel() {
		if (this.commentsLabel == null) {
			this.commentsLabel = new JLabel();
			this.commentsLabel.setText("Comments: ");
		}
		return this.commentsLabel;
	}

	private JTextArea getCommentsTextArea() {
		if (this.commentsTextArea == null) {
			this.commentsTextArea = new JTextArea();
			//this.commentsTextArea.setPreferredSize(new Dimension(200,100));
			this.commentsTextArea.addFocusListener(new FocusListener() {
				public void focusGained(FocusEvent e) {
					// ignore this
				}

				public void focusLost(FocusEvent e) {
					Gui.this.currentTask.setComments(((JTextArea) e.getSource()).getText());
					logger.info("Comments Changed to: " + ((JTextArea) e.getSource()).getText());
				}
			});
		}
		return this.commentsTextArea;
	}

	private JScrollPane getCommentsTextAreaScrollPane() {
		if (this.commentsTextAreaScrollPane == null) {
			this.commentsTextAreaScrollPane = new JScrollPane(getCommentsTextArea());
			this.commentsTextAreaScrollPane.setPreferredSize(new Dimension(200, 100));
		}
		return this.commentsTextAreaScrollPane;
	}

	private JLabel getCostLabel() {
		if (this.costLabel == null) {
			this.costLabel = new JLabel();
			this.costLabel.setText("Cost: ");
		}
		return this.costLabel;
	}

	private JTextField getCostTextField() {
		if (this.costTextField == null) {
			this.costTextField = new JTextField();
			this.costTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setCost(Double.parseDouble(((JTextField) e.getSource()).getText()));
					logger.info("Set cost to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.costTextField;
	}

	private JDateChooser getCreationDateChooser() {
		if (this.creationDateChooser == null) {
			this.creationDateChooser = new JDateChooser();
		}
		return this.creationDateChooser;
	}

	private JLabel getCreationDateLabel() {
		if (this.creationDateLabel == null) {
			this.creationDateLabel = new JLabel();
			this.creationDateLabel.setText("Creation Date: ");
		}
		return this.creationDateLabel;
	}

	private JMenuItem getDefaultLookAndFeelItem() {
		if (this.defaultLookAndFeelItem == null) {
			this.defaultLookAndFeelItem = new JMenuItem();
			this.defaultLookAndFeelItem.setText("Default");
			this.defaultLookAndFeelItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
						SwingUtilities.updateComponentTreeUI(Gui.this.getJFrame());
					} catch (ClassNotFoundException e1) {
						logger.error("ClassNotFoundException", e1);
					} catch (InstantiationException e1) {
						logger.error("InstantiationException", e1);
					} catch (IllegalAccessException e1) {
						logger.error("IllegalAccessException", e1);
					} catch (UnsupportedLookAndFeelException e1) {
						logger.error("UnsupportedLookAndFeelException", e1);
					}
				}
			});
		}
		return this.defaultLookAndFeelItem;
	}

	private JLabel getDependsLabel() {
		if (this.dependsLabel == null) {
			this.dependsLabel = new JLabel();
			this.dependsLabel.setText("Depends: ");
		}
		return this.dependsLabel;
	}

	private JTextField getDependsTextField() {
		if (this.dependsTextField == null) {
			this.dependsTextField = new JTextField();
			this.dependsTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setDepends(((JTextField) e.getSource()).getText());
					logger.info("Set Depends to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.dependsTextField;
	}

	private JDateChooser getDoneDateChooser() {
		if (this.doneDateChooser == null) {
			this.doneDateChooser = new JDateChooser();
		}
		return this.doneDateChooser;
	}

	private JLabel getDoneDateLabel() {
		if (this.doneDateLabel == null) {
			this.doneDateLabel = new JLabel();
			this.doneDateLabel.setText("Done date: ");
		}
		return this.doneDateLabel;
	}

	private JDateChooser getDueDateChooser() {
		if (this.dueDateChooser == null) {
			this.dueDateChooser = new JDateChooser();
		}
		return this.dueDateChooser;
	}

	private JLabel getDueDateLabel() {
		if (this.dueDateLabel == null) {
			this.dueDateLabel = new JLabel();
			this.dueDateLabel.setText("Due Date: ");
		}
		return this.dueDateLabel;
	}

	/**
	 * This method initializes jMenu
	 *
	 * @return javax.swing.JMenu
	 */
	private JMenu getEditMenu() {
		if (editMenu == null) {
			editMenu = new JMenu();
			editMenu.setText("Edit");
			editMenu.add(getAddSubnodeItem());
			editMenu.add(getRemoveSubnodeItem());
			// TODO Add "remove completed"
		}
		return editMenu;
	}

	/**
	 * This method initializes jMenuItem
	 *
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getExitMenuItem() {
		if (exitMenuItem == null) {
			exitMenuItem = new JMenuItem();
			exitMenuItem.setText("Exit");
			exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK, true));
			exitMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			});
		}
		return exitMenuItem;
	}

	private JMenuItem getExportAsCSVMenuItem() {
		if (this.exportAsCSVMenuItem == null) {
			this.exportAsCSVMenuItem = new JMenuItem();
			this.exportAsCSVMenuItem.setText("CSV");
			this.exportAsCSVMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.error("NOT YET IMPLEMENTED! - Exporting as CSV to: " + exportFile);
						//IExport export = new CSVExport();
						//export.setOutputFile(exportFile);
						//export.setTodoList(Gui.this.tdl);
						//export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsCSVMenuItem;
	}

	private JMenuItem getExportAsHTMLMenuItem() {
		if (this.exportAsHTMLMenuItem == null) {
			this.exportAsHTMLMenuItem = new JMenuItem();
			this.exportAsHTMLMenuItem.setText("HTML");
			this.exportAsHTMLMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.info("Exporting as HTML to: " + exportFile);
						IExport export = new HTMLExport();
						export.setOutputFile(exportFile);
						export.setTodoList(Gui.this.tdl);
						export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsHTMLMenuItem;
	}

	private JMenuItem	viewAsPriorityTreemapItem	= null;

	private JMenuItem getViewAsPriorityTreemapItem() {
		if (this.viewAsPriorityTreemapItem == null) {
			this.viewAsPriorityTreemapItem = new JMenuItem();
			this.viewAsPriorityTreemapItem.setText("Treemap (by Priority)");
			this.viewAsPriorityTreemapItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TreemapPriorityView tmv = new TreemapPriorityView();
					logger.info("Todolist: " + Gui.this.tdl.getFileversion());
					tmv.setTodolist(Gui.this.tdl);
					Gui.this.getJSplitPane().setRightComponent(tmv.getView());
				}
			});
		}
		return this.viewAsPriorityTreemapItem;
	}

	private JMenuItem viewAsRiskTreemapItem = null;

	private JMenuItem getViewAsRiskTreemapItem() {
		if(this.viewAsRiskTreemapItem == null) {
			this.viewAsRiskTreemapItem = new JMenuItem();
			this.viewAsRiskTreemapItem.setText("Treemap (by Risk)");
			this.viewAsRiskTreemapItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					TreemapRiskView tmv = new TreemapRiskView();
					logger.info("Todolist: " + Gui.this.tdl.getFileversion());
					tmv.setTodolist(Gui.this.tdl);
					Gui.this.getJSplitPane().setRightComponent(tmv.getView());
				}
			});
		}
		return this.viewAsRiskTreemapItem;
	}

	private JMenuItem getExportAsLatexMenuItem() {
		if (this.exportAsLatexMenuItem == null) {
			this.exportAsLatexMenuItem = new JMenuItem();
			this.exportAsLatexMenuItem.setText("LaTeX");
			this.exportAsLatexMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.error("NOT YET IMPLEMENTED! - Exporting as Latex to: " + exportFile);
						//IExport export = new LatexExport();
						//export.setOutputFile(exportFile);
						//export.setTodoList(Gui.this.tdl);
						//export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsLatexMenuItem;
	}

	private JMenuItem getExportAsPDFMenuItem() {
		if (this.exportAsPDFMenuItem == null) {
			this.exportAsPDFMenuItem = new JMenuItem();
			this.exportAsPDFMenuItem.setText("PDF");
			this.exportAsPDFMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.error("NOT YET IMPLEMENTED! - Exporting as PDF to: " + exportFile);
						//IExport export = new PDFExport();
						//export.setOutputFile(exportFile);
						//export.setTodoList(Gui.this.tdl);
						//export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsPDFMenuItem;
	}

	private JMenuItem getExportAsTextMenuItem() {
		if (this.exportAsTextMenuItem == null) {
			this.exportAsTextMenuItem = new JMenuItem();
			this.exportAsTextMenuItem.setText("Text");
			this.exportAsTextMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.info("Saving to: " + exportFile);
						IExport export = new TextExport();
						export.setOutputFile(exportFile);
						export.setTodoList(Gui.this.tdl);
						export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsTextMenuItem;
	}

	private JMenuItem getExportAsXMLMenuItem() {
		if (this.exportAsXMLMenuItem == null) {
			this.exportAsXMLMenuItem = new JMenuItem();
			this.exportAsXMLMenuItem.setText("XML");
			this.exportAsXMLMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File exportFile = fc.getSelectedFile();
						logger.info("Exporting as XML to: " + exportFile);
						IExport export = new XMLExport();
						export.setOutputFile(exportFile);
						export.setTodoList(Gui.this.tdl);
						export.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.exportAsXMLMenuItem;
	}

	private JMenu getExportSubMenu() {
		if (this.exportSubMenu == null) {
			this.exportSubMenu = new JMenu();
			this.exportSubMenu.setText("Export as ...");
			this.exportSubMenu.add(getExportAsCSVMenuItem());
			this.exportSubMenu.add(getExportAsHTMLMenuItem());
			this.exportSubMenu.add(getExportAsLatexMenuItem());
			this.exportSubMenu.add(getExportAsPDFMenuItem());
			this.exportSubMenu.add(getExportAsTextMenuItem());
			this.exportSubMenu.add(getExportAsXMLMenuItem());
		}
		return this.exportSubMenu;
	}

	private JLabel getExternalIdLabel() {
		if (this.externalIdLabel == null) {
			this.externalIdLabel = new JLabel();
			this.externalIdLabel.setText("External ID: ");
		}
		return this.externalIdLabel;
	}

	private JTextField getExternalIdTextField() {
		if (this.externalIdTextField == null) {
			this.externalIdTextField = new JTextField();
			this.externalIdTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setExternalId(((JTextField) e.getSource()).getText());
					logger.info("Set external ID to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.externalIdTextField;
	}

	/**
	 * This method initializes jMenu
	 *
	 * @return javax.swing.JMenu
	 */
	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.add(getNewMenuItem());
			fileMenu.add(getOpenMenuItem());
			fileMenu.add(getSaveMenuItem());
			fileMenu.add(getSaveToMenuItem());
			fileMenu.addSeparator();
			fileMenu.add(getImportSubMenu());
			fileMenu.add(getExportSubMenu());
			fileMenu.addSeparator();
			fileMenu.add(getCloseMenuItem());
			fileMenu.add(getExitMenuItem());
		}
		return fileMenu;
	}

	private JLabel getFileRefPathLabel() {
		if (this.fileRefPathLabel == null) {
			this.fileRefPathLabel = new JLabel();
			this.fileRefPathLabel.setText("Reference File: ");
		}
		return this.fileRefPathLabel;
	}

	private JTextField getFileRefPathTextField() {
		if (this.fileRefPathTextField == null) {
			this.fileRefPathTextField = new JTextField();
			this.fileRefPathTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setFileReferingPath(((JTextField) e.getSource()).getText());
					logger.info("Set external ID to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.fileRefPathTextField;
	}

	private GridBagConstraints getGridBagConstraints() {
		if (this.gridBagConstraints == null) {
			this.gridBagConstraints = new GridBagConstraints();
		}
		return this.gridBagConstraints;
	}

	private GridBagLayout getGridBagLayout() {
		if (this.gridBagLayout == null) {
			this.gridBagLayout = new GridBagLayout();
		}
		return this.gridBagLayout;
	}

	/**
	 * This method initializes jMenu
	 *
	 * @return javax.swing.JMenu
	 */
	private JMenu getHelpMenu() {
		if (helpMenu == null) {
			helpMenu = new JMenu();
			helpMenu.setText("Help");
			helpMenu.add(getAboutMenuItem());
		}
		return helpMenu;
	}

	private JMenuItem getImportFromXMLMenuItem() {
		if (this.importFromXMLMenuItem == null) {
			this.importFromXMLMenuItem = new JMenuItem();
			this.importFromXMLMenuItem.setText("XML");
			this.importFromXMLMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showOpenDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						File importFile = fc.getSelectedFile();
						logger.info("NOT YET IMPLEMENTED! - Importing from XML to: " + importFile);
						//IImport importer = new XMLImport();
						//importer.setOutputFile(importFile);
						//importer.setTodoList(Gui.this.tdl);
						//importer.export();
					} else {
						logger.info("The user canceled the save dialog");
					}
				}
			});
		}
		return this.importFromXMLMenuItem;
	}

	private JMenu getImportSubMenu() {
		if (this.importSubMenu == null) {
			this.importSubMenu = new JMenu();
			this.importSubMenu.setText("Import from ...");
			this.importSubMenu.add(getImportFromXMLMenuItem());
		}
		return this.importSubMenu;
	}

	/**
	 * This method initializes jContentPane
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BoxLayout(getJContentPane(), BoxLayout.X_AXIS));
			jContentPane.add(getJSplitPane()); // TODO Refactor
		}
		return jContentPane;
	}

	/**
	 * This method initializes jFrame
	 *
	 * @return javax.swing.JFrame
	 */
	private JFrame getJFrame() {
		if (jFrame == null) {
			jFrame = new JFrame();
			jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			jFrame.setJMenuBar(getJJMenuBar());
			jFrame.setSize(800, 600);
			jFrame.setContentPane(getJContentPane());
			jFrame.setTitle("jTodoList");
		}
		return jFrame;
	}

	/**
	 * This method initializes jJMenuBar
	 *
	 * @return javax.swing.JMenuBar
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getFileMenu());
			jJMenuBar.add(getEditMenu());
			jJMenuBar.add(getViewMenu());
			jJMenuBar.add(getLafMenu());
			jJMenuBar.add(getHelpMenu());
		}
		return jJMenuBar;
	}

	private JList getJList() {
		if (jList == null) {
			jList = new JList(this.currentTaskList);
			jList.addMouseListener(new MouseListener() {

				public void mouseClicked(MouseEvent e) {
					int index = Gui.this.jList.locationToIndex(e.getPoint());
					logger.trace("Clicked on Item " + index);
					Gui.this.displayTask(Gui.this.currentTaskList[index]);
				}

				public void mouseEntered(MouseEvent e) {
					// not necessary
				}

				public void mouseExited(MouseEvent e) {
					// not necessary
				}

				public void mousePressed(MouseEvent e) {
					// not necessary
				}

				public void mouseReleased(MouseEvent e) {
					// not necessary
				}

			});
		}
		return jList;
	}

	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			logger.info("getJScrollPane() - Generation new Scroll Pane");
			jScrollPane = new JScrollPane(getJTree());
		}
		return jScrollPane;
	}

	private JScrollPane getJScrollPaneList() {
		jScrollPane = new JScrollPane(getJList());
		return jScrollPane;
	}

	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, getJScrollPane(), getTaskDetailsPane());
			jSplitPane.setDividerLocation(200);
			jSplitPane.setDividerSize(5);
			jSplitPane.setDoubleBuffered(true);
		}
		return jSplitPane;
	}

	/**
	 * This method initializes jTextPane
	 *
	 * @return javax.swing.JTextPane
	 */
	private JTextPane getJTextPane() {
		if (jTextPane == null) {
			jTextPane = new JTextPane();
			jTextPane.setText("Test");
		}
		return jTextPane;
	}

	/**
	 * This method initializes jTree
	 *
	 * @return javax.swing.JTree
	 */
	private JTree getJTree() {
		if (jTree == null) {
			logger.info("getJTree() - Generating new Tree");
			jTree = this.tdl.getAsTree();
			jTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			jTree.addTreeSelectionListener(new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) Gui.this.jTree
							.getLastSelectedPathComponent();

					if (node == null)
						return;

					Object nodeInfo = node.getUserObject();

					if (node.isLeaf()) {
						ITask t = (ITask) nodeInfo;
						Gui.this.displayTask(t);
					} else {
						ITask t = (ITask) nodeInfo;
						Gui.this.displayTask(t);
					}
				}
			});
		}
		return jTree;
	}

	private JMenu getLafMenu() {
		if (this.lookAndFeelMenu == null) {
			this.lookAndFeelMenu = new JMenu();
			this.lookAndFeelMenu.setText("Look and Feel");
			this.lookAndFeelMenu.add(getSystemLookAndFeelItem());
			this.lookAndFeelMenu.add(getDefaultLookAndFeelItem());
			this.lookAndFeelMenu.add(getMotifLookAndFeelItem());
			this.lookAndFeelMenu.add(getPlasticLookAndFeelItem());
		}
		return this.lookAndFeelMenu;
	}

	private JLabel getLastModDateLabel() {
		if (this.lastModDateLabel == null) {
			this.lastModDateLabel = new JLabel();
			this.lastModDateLabel.setText(" ");
		}
		return this.lastModDateLabel;
	}

	private JLabel getLastModLabel() {
		if (this.lastModLabel == null) {
			this.lastModLabel = new JLabel();
			this.lastModLabel.setText("Last Modification: ");
		}
		return this.lastModLabel;
	}

	private JMenuItem getMotifLookAndFeelItem() {
		if (this.motifLookAndFeelItem == null) {
			this.motifLookAndFeelItem = new JMenuItem();
			this.motifLookAndFeelItem.setText("Motif");
			this.motifLookAndFeelItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
						SwingUtilities.updateComponentTreeUI(Gui.this.getJFrame());
					} catch (ClassNotFoundException e1) {
						logger.error("ClassNotFoundException", e1);
					} catch (InstantiationException e1) {
						logger.error("InstantiationException", e1);
					} catch (IllegalAccessException e1) {
						logger.error("IllegalAccessException", e1);
					} catch (UnsupportedLookAndFeelException e1) {
						logger.error("UnsupportedLookAndFeelException", e1);
					}
				}
			});
		}
		return this.motifLookAndFeelItem;
	}

	private JMenuItem getNewMenuItem() {
		if (this.newMenuItem == null) {
			this.newMenuItem = new JMenuItem();
			this.newMenuItem.setText("New");
			this.newMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK, true));
			this.newMenuItem.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					Gui.this.tdl = new TodoList("ToDoList");
					Gui.this.tdl.addTask(new Task("a Subtask"));
					Gui.this.refreshJTree();
				}

			});
		}
		return newMenuItem;
	}

	private JMenuItem getOpenMenuItem() {
		if (openMenuItem == null) {
			this.openMenuItem = new JMenuItem();
			this.openMenuItem.setText("Open ...");
			this.openMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK, true));
			this.openMenuItem.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showOpenDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						Gui.this.openSaveFile = fc.getSelectedFile();
						logger.info("Opening file: " + Gui.this.openSaveFile);
						Gui.this.readFromFile(Gui.this.openSaveFile);

					} else {
						logger.info("The user canceled the open dialog");
					}
				}

			});
		}
		return openMenuItem;
	}

	private JLabel getPercentDoneLabel() {
		if (this.percentDoneLabel == null) {
			this.percentDoneLabel = new JLabel();
			this.percentDoneLabel.setText("Percent done: ");
		}
		return this.percentDoneLabel;
	}

	private JSlider getPercentDoneSlider() {
		if (this.percentDoneSlider == null) {
			this.percentDoneSlider = new JSlider();
			this.percentDoneSlider.setMinimum(0);
			this.percentDoneSlider.setMaximum(100);
			this.percentDoneSlider.setValue(0);
			this.percentDoneSlider.setMinorTickSpacing(5);
			this.percentDoneSlider.setMajorTickSpacing(10);
			this.percentDoneSlider.setSnapToTicks(true);
			this.percentDoneSlider.setPaintTicks(true);
			this.percentDoneSlider.setPaintTrack(true);
			this.percentDoneSlider.setPaintLabels(true);
			this.percentDoneSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					Gui.this.currentTask.setPercentdone(((JSlider) e.getSource()).getValue());
					logger.info("Set Percentdone to: " + ((JSlider) e.getSource()).getValue());
				}
			});
		}
		return this.percentDoneSlider;
	}

	private JLabel getPersonLabel() {
		if (this.personLabel == null) {
			this.personLabel = new JLabel();
			this.personLabel.setText("Person: ");
		}
		return this.personLabel;
	}

	private JTextField getPersonTextField() {
		if (this.personTextField == null) {
			this.personTextField = new JTextField();
			this.personTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setStatus(((JTextField) e.getSource()).getText());
					logger.info("Set Person to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.personTextField;
	}

	private JMenuItem getPlasticLookAndFeelItem() {
		if (this.plasticLookAndFeelItem == null) {
			this.plasticLookAndFeelItem = new JMenuItem();
			this.plasticLookAndFeelItem.setText("Plastic");
			this.plasticLookAndFeelItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						UIManager.setLookAndFeel("com.jgoodies.looks.plastic.PlasticLookAndFeel");
						SwingUtilities.updateComponentTreeUI(Gui.this.getJFrame());
					} catch (ClassNotFoundException e1) {
						logger.error("ClassNotFoundException", e1);
					} catch (InstantiationException e1) {
						logger.error("InstantiationException", e1);
					} catch (IllegalAccessException e1) {
						logger.error("IllegalAccessException", e1);
					} catch (UnsupportedLookAndFeelException e1) {
						logger.error("UnsupportedLookAndFeelException", e1);
					}
				}
			});
		}
		return this.plasticLookAndFeelItem;
	}

	private JLabel getPriorityLabel() {
		if (this.priorityLabel == null) {
			this.priorityLabel = new JLabel();
			this.priorityLabel.setText("Priority: ");
		}
		return this.priorityLabel;
	}

	private JSlider getPrioritySlider() {
		if (this.prioritySlider == null) {
			this.prioritySlider = new JSlider();
			this.prioritySlider.setMinimum(0);
			this.prioritySlider.setMaximum(10);
			this.prioritySlider.setValue(0);
			this.prioritySlider.setMinorTickSpacing(1);
			this.prioritySlider.setMajorTickSpacing(2);
			this.prioritySlider.setSnapToTicks(true);
			this.prioritySlider.setPaintTicks(true);
			this.prioritySlider.setPaintTrack(true);
			this.prioritySlider.setPaintLabels(true);
			this.prioritySlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					Gui.this.currentTask.setPriority(((JSlider) e.getSource()).getValue());
					logger.info("Set Priority to: " + ((JSlider) e.getSource()).getValue());
				}
			});
		}
		return this.prioritySlider;
	}

	private JMenuItem getRemoveSubnodeItem() {
		if (removeSubnodeItem == null) {
			removeSubnodeItem = new JMenuItem();
			removeSubnodeItem.setText("Remove Subnode");
			removeSubnodeItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, Event.CTRL_MASK, true));
			removeSubnodeItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (Gui.this.tdl.remove(Gui.this.currentTask)) {
						Gui.this.refreshJTree();
						logger.info("Should have removed a Task");
					} else {
						logger.info("Task not found");
					}

				}
			});
		}
		return removeSubnodeItem;
	}

	private JLabel getRiskLabel() {
		if (this.riskLabel == null) {
			this.riskLabel = new JLabel();
			this.riskLabel.setText("Risk: ");
		}
		return this.riskLabel;
	}

	private JSlider getRiskSlider() {
		if (this.riskSlider == null) {
			this.riskSlider = new JSlider();
			this.riskSlider.setMinimum(0);
			this.riskSlider.setMaximum(10);
			this.riskSlider.setValue(0);
			this.riskSlider.setMinorTickSpacing(1);
			this.riskSlider.setMajorTickSpacing(2);
			this.riskSlider.setSnapToTicks(true);
			this.riskSlider.setPaintTicks(true);
			this.riskSlider.setPaintTrack(true);
			this.riskSlider.setPaintLabels(true);
			this.riskSlider.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					Gui.this.currentTask.setRisk(((JSlider) e.getSource()).getValue());
					logger.info("Set Risk to: " + ((JSlider) e.getSource()).getValue());
				}
			});
		}
		return this.riskSlider;
	}

	/**
	 * This method initializes jMenuItem
	 *
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getSaveMenuItem() {
		if (saveMenuItem == null) {
			saveMenuItem = new JMenuItem();
			saveMenuItem.setText("Save");
			saveMenuItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					if (Gui.this.openSaveFile == null || !Gui.this.openSaveFile.canWrite()) {
						final JFileChooser fc = new JFileChooser();
						int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
						if (returnValue == JFileChooser.APPROVE_OPTION) {
							Gui.this.openSaveFile = fc.getSelectedFile();
							logger.info("Saving to: " + Gui.this.openSaveFile);
							Gui.this.saveToFile(Gui.this.openSaveFile);
						} else {
							logger.info("The user canceled the save dialog");
						}
					} else {
						logger.info("Saving to: " + Gui.this.openSaveFile);
						Gui.this.saveToFile(Gui.this.openSaveFile);
					}
				}
			});
			saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK, true));
		}
		return saveMenuItem;
	}

	private JMenuItem getSaveToMenuItem() {
		if (saveToMenuItem == null) {
			this.saveToMenuItem = new JMenuItem();
			this.saveToMenuItem.setText("Save to ...");
			this.saveToMenuItem.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					Gui.this.saveCurrentTask();
					final JFileChooser fc = new JFileChooser();
					int returnValue = fc.showSaveDialog(Gui.this.getJFrame());
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						Gui.this.openSaveFile = fc.getSelectedFile();
						logger.info("Saving to: " + Gui.this.openSaveFile);
						Gui.this.saveToFile(Gui.this.openSaveFile);
					} else {
						logger.info("The user canceled the save dialog");
					}
				}

			});
		}
		return saveToMenuItem;
	}

	private JDateChooser getStartDateChooser() {
		if (this.startDateChooser == null) {
			this.startDateChooser = new JDateChooser();
		}
		return this.startDateChooser;
	}

	private JLabel getStartDateLabel() {
		if (startDateLabel == null) {
			this.startDateLabel = new JLabel();
			this.startDateLabel.setText("Start Date: ");
		}
		return startDateLabel;
	}

	private JLabel getStatusLabel() {
		if (this.statusLabel == null) {
			this.statusLabel = new JLabel();
			this.statusLabel.setText("Status: ");
		}
		return this.statusLabel;
	}

	private JTextField getStatusTextField() {
		if (this.statusTextField == null) {
			this.statusTextField = new JTextField();
			this.statusTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setStatus(((JTextField) e.getSource()).getText());
					logger.info("Set Status to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.statusTextField;
	}

	private JMenuItem getSystemLookAndFeelItem() {
		if (this.systemLookAndFeelItem == null) {
			this.systemLookAndFeelItem = new JMenuItem();
			this.systemLookAndFeelItem.setText("System");
			this.systemLookAndFeelItem.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					try {
						UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
						SwingUtilities.updateComponentTreeUI(Gui.this.getJFrame());
					} catch (ClassNotFoundException e1) {
						logger.error("ClassNotFoundException", e1);
					} catch (InstantiationException e1) {
						logger.error("InstantiationException", e1);
					} catch (IllegalAccessException e1) {
						logger.error("IllegalAccessException", e1);
					} catch (UnsupportedLookAndFeelException e1) {
						logger.error("UnsupportedLookAndFeelException", e1);
					}
				}
			});
		}
		return this.systemLookAndFeelItem;
	}

	private JPanel getTaskDetailsPane() {
		if (jTaskDetailPane == null) {
			jTaskDetailPane = new JPanel();
		}
		return jTaskDetailPane;
	}

	/**
	 * When swtiching the task this method updates the right side of the split pane.
	 * @link file:///usr/local/java/jdk1.6.0/docs/api/java/awt/GridBagLayout.html
	 * @param t
	 * @return
	 */
	private JPanel getTaskDetailsPane(ITask t) {
		saveCurrentTask();
		jTaskDetailPane.removeAll();
		jTaskDetailPane = new JPanel();
		currentTask = t;
		GridBagLayout gridbag = getGridBagLayout();
		GridBagConstraints c = getGridBagConstraints();
		jTaskDetailPane.setLayout(gridbag);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.gridwidth = 1;
		// Title
		getGridBagLayout().setConstraints(getTitleLabel(), c);
		jTaskDetailPane.add(getTitleLabel());
		getTitleTextField().setText(t.getTitle());
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(getTitleTextField(), c);
		jTaskDetailPane.add(getTitleTextField());
		// Priority
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getPriorityLabel(), c);
		jTaskDetailPane.add(getPriorityLabel());
		getPrioritySlider().setValue(t.getPriority());
		//c.gridwidth = GridBagConstraints.REMAINDER; // end row
		gridbag.setConstraints(getPrioritySlider(), c);
		jTaskDetailPane.add(getPrioritySlider());
		// ### Risk
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getRiskLabel(), c);
		jTaskDetailPane.add(getRiskLabel());
		getRiskSlider().setValue(t.getRisk());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getRiskSlider().setValue(t.getRisk());
		gridbag.setConstraints(getRiskSlider(), c);
		jTaskDetailPane.add(getRiskSlider());
		// Percentdone
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getPercentDoneLabel(), c);
		jTaskDetailPane.add(getPercentDoneLabel());
		getPercentDoneSlider().setValue(t.getPercentdone());
		c.gridwidth = GridBagConstraints.REMAINDER;
		gridbag.setConstraints(getPercentDoneSlider(), c);
		jTaskDetailPane.add(getPercentDoneSlider());
		// ### Start Date
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getStartDateLabel(), c);
		jTaskDetailPane.add(getStartDateLabel());
		//c.gridwidth = GridBagConstraints.REMAINDER;
		getStartDateChooser().setDate(t.getStartDate());
		gridbag.setConstraints(getStartDateChooser(), c);
		jTaskDetailPane.add(getStartDateChooser());
		// Done Date (label only)
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getDoneDateLabel(), c);
		jTaskDetailPane.add(getDoneDateLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getDoneDateChooser().setDate(t.getDoneDate());
		gridbag.setConstraints(getDoneDateChooser(), c);
		jTaskDetailPane.add(getDoneDateChooser());
		// last mod date (label only)
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getLastModLabel(), c);
		jTaskDetailPane.add(getLastModLabel());
		//c.gridwidth = GridBagConstraints.REMAINDER;
		getLastModDateLabel().setText(t.getLastModifiedDate().toString());
		gridbag.setConstraints(getLastModDateLabel(), c);
		jTaskDetailPane.add(getLastModDateLabel());
		// Due Date
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getDueDateLabel(), c);
		jTaskDetailPane.add(getDueDateLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getDueDateChooser().setDate(t.getDueDate());
		gridbag.setConstraints(getDueDateChooser(), c);
		jTaskDetailPane.add(getDueDateChooser());
		// Status
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getStatusLabel(), c);
		jTaskDetailPane.add(getStatusLabel());
		//c.gridwidth = GridBagConstraints.REMAINDER;
		getStatusTextField().setText(t.getStatus());
		gridbag.setConstraints(getStatusTextField(), c);
		jTaskDetailPane.add(getStatusTextField());
		// Allocated By
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getAllocatedByLabel(), c);
		jTaskDetailPane.add(getAllocatedByLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getAllocatedByTextField().setText(t.getAllocatedBy());
		gridbag.setConstraints(getAllocatedByTextField(), c);
		jTaskDetailPane.add(getAllocatedByTextField());
		// Person
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getPersonLabel(), c);
		jTaskDetailPane.add(getPersonLabel());
		//c.gridwidth = GridBagConstraints.REMAINDER;
		getPersonTextField().setText(t.getPerson());
		gridbag.setConstraints(getPersonTextField(), c);
		jTaskDetailPane.add(getPersonTextField());
		// Depends
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getDependsLabel(), c);
		jTaskDetailPane.add(getDependsLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getDependsTextField().setText(t.getDepends());
		gridbag.setConstraints(getDependsTextField(), c);
		jTaskDetailPane.add(getDependsTextField());
		// ### Creation Date (label only) // FIXME This should be read only!
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getCreationDateLabel(), c);
		jTaskDetailPane.add(getCreationDateLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		//getCreationDateTextField().setText(t.getCreationDate().toString());
		getCreationDateChooser().setDate(t.getCreationDate());
		//gridbag.setConstraints(getCreationDateTextField(), c);
		gridbag.setConstraints(getCreationDateChooser(), c);
		//jTaskDetailPane.add(getCreationDateTextField());
		jTaskDetailPane.add(getCreationDateChooser());
		// ### Time spent (+ units, + watch(stopuhr))
		// TODO Add stop-watch
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getTimeSpentLabel(), c);
		jTaskDetailPane.add(getTimeSpentLabel());
		getTimeSpentTextField().setText(new Double(t.getTimeSpent()).toString());
		gridbag.setConstraints(getTimeSpentTextField(), c);
		jTaskDetailPane.add(getTimeSpentTextField());
		gridbag.setConstraints(getTimeSpentUnitsLabel(), c);
		jTaskDetailPane.add(getTimeSpentUnitsLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getTimeSpentUnitsTextField().setText(t.getTimeSpentUnits());
		gridbag.setConstraints(getTimeSpentUnitsTextField(), c);
		jTaskDetailPane.add(getTimeSpentUnitsTextField());
		// ### external id
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getExternalIdLabel(), c);
		jTaskDetailPane.add(getExternalIdLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getExternalIdTextField().setText(t.getExternalId());
		gridbag.setConstraints(getExternalIdTextField(), c);
		jTaskDetailPane.add(getExternalIdTextField());
		// ### timeestimate (+units)
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getTimeEstLabel(), c);
		jTaskDetailPane.add(getTimeEstLabel());
		getTimeEstTextField().setText(new Double(t.getTimeEstimated()).toString());
		gridbag.setConstraints(getTimeEstTextField(), c);
		jTaskDetailPane.add(getTimeEstTextField());
		gridbag.setConstraints(getTimeEstUnitsLabel(), c);
		jTaskDetailPane.add(getTimeEstUnitsLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getTimeEstUnitsTextField().setText(t.getTimeEstimatedUnits());
		gridbag.setConstraints(getTimeEstUnitsTextField(), c);
		jTaskDetailPane.add(getTimeEstUnitsTextField());
		// ### category
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getCategoryLabel(), c);
		jTaskDetailPane.add(getCategoryLabel());
		//c.gridwidth = GridBagConstraints.REMAINDER;
		getCategoryTextField().setText(t.getCategory());
		gridbag.setConstraints(getCategoryTextField(), c);
		jTaskDetailPane.add(getCategoryTextField());
		// ### cost
		//c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getCostLabel(), c);
		jTaskDetailPane.add(getCostLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getCostTextField().setText(new Double(t.getCost()).toString());
		gridbag.setConstraints(getCostTextField(), c);
		jTaskDetailPane.add(getCostTextField());
		// ### filerefpath (+file chooser)
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getFileRefPathLabel(), c);
		jTaskDetailPane.add(getFileRefPathLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getFileRefPathTextField().setText(t.getFileReferingPath());
		gridbag.setConstraints(getFileRefPathTextField(), c);
		jTaskDetailPane.add(getFileRefPathTextField());
		// ### comments (textarea)
		c.gridwidth = 1; // reset to default
		gridbag.setConstraints(getCommentsLabel(), c);
		jTaskDetailPane.add(getCommentsLabel());
		c.gridwidth = GridBagConstraints.REMAINDER;
		getCommentsTextArea().setText(t.getComments());
		gridbag.setConstraints(getCommentsTextAreaScrollPane(), c);
		jTaskDetailPane.add(getCommentsTextAreaScrollPane());
		// ### validate/update the graphics component
		jTaskDetailPane.validate();
		return jTaskDetailPane;
	}

	private JLabel getTimeEstLabel() {
		if (this.timeEstLabel == null) {
			this.timeEstLabel = new JLabel();
			this.timeEstLabel.setText("Time Estimated: ");
		}
		return this.timeEstLabel;
	}

	private JTextField getTimeEstTextField() {
		if (this.timeEstTextField == null) {
			this.timeEstTextField = new JTextField();
			this.timeEstTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setTimeEstimated(Double.parseDouble(((JTextField) e.getSource()).getText()));
					logger.info("Set time spent to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.timeEstTextField;
	}

	private JLabel getTimeEstUnitsLabel() {
		if (this.timeEstUnitsLabel == null) {
			this.timeEstUnitsLabel = new JLabel();
			this.timeEstUnitsLabel.setText("Units: ");
		}
		return this.timeEstUnitsLabel;
	}

	private JTextField getTimeEstUnitsTextField() {
		if (this.timeEstUnitsTextField == null) {
			this.timeEstUnitsTextField = new JTextField();
			this.timeEstUnitsTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setTimeEstimatedUnits(((JTextField) e.getSource()).getText());
					logger.info("Set time est units to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.timeEstUnitsTextField;
	}

	private JLabel getTimeSpentLabel() {
		if (this.timeSpentLabel == null) {
			this.timeSpentLabel = new JLabel();
			this.timeSpentLabel.setText("Time spent: ");
		}
		return this.timeSpentLabel;
	}

	private JTextField getTimeSpentTextField() {
		if (this.timeSpentTextField == null) {
			this.timeSpentTextField = new JTextField();
			this.timeSpentTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setTimeSpent(Double.parseDouble(((JTextField) e.getSource()).getText()));
					logger.info("Set time spent to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.timeSpentTextField;
	}

	private JLabel getTimeSpentUnitsLabel() {
		if (this.timeSpentUnitsLabel == null) {
			this.timeSpentUnitsLabel = new JLabel();
			this.timeSpentUnitsLabel.setText("Units: ");
		}
		return this.timeSpentUnitsLabel;
	}

	private JTextField getTimeSpentUnitsTextField() {
		if (this.timeSpentUnitsTextField == null) {
			this.timeSpentUnitsTextField = new JTextField();
			this.timeSpentUnitsTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setTimeSpentUnits(((JTextField) e.getSource()).getText());
					logger.info("Set time est units to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.timeSpentUnitsTextField;
	}

	private JLabel getTitleLabel() {
		if (this.titleLabel == null) {
			this.titleLabel = new JLabel();
			this.titleLabel.setText("Title: ");
		}
		return this.titleLabel;
	}

	private JTextField getTitleTextField() {
		if (this.titleTextField == null) {
			this.titleTextField = new JTextField();
			this.titleTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Gui.this.currentTask.setTitle(((JTextField) e.getSource()).getText());
					logger.info("Set Title to: " + ((JTextField) e.getSource()).getText());
				}
			});
		}
		return this.titleTextField;
	}

	private JMenu getViewMenu() {
		if (viewMenu == null) {
			viewMenu = new JMenu();
			viewMenu.setText("View");
			viewMenu.add(getAsTreeItem());
			viewMenu.add(getAsListByDueDateItem());
			viewMenu.add(getAsListByPriorityItem());
			viewMenu.add(getAsListByRiskItem());
			viewMenu.add(getViewAsPriorityTreemapItem());
			viewMenu.add(getViewAsRiskTreemapItem());
			// TODO Add "show uncomplete" (checkbox)
			// TODO Add "show complete" (checkbox)
		}
		return viewMenu;
	}

	/**
	 * Reads an Todo-List from an XML file and set this as active.
	 * @param file
	 */
	private void readFromFile(File file) {
		try {
			SAXBuilder sax = new SAXBuilder();
			Document xml = sax.build(file);
			this.openSaveFile = file;
			this.tdl = null;
			this.tdl = new TodoList("ToDoList");
			this.tdl.setXML(xml);
			this.refreshJTree();
		} catch (Exception e) {
			logger.info("Got Exception", e);
			SAXBuilder sax = new SAXBuilder();
			Document xml = null;
			try {
				xml = sax.build(this.sanitizeFile(file));
			} catch (JDOMException e1) {
				logger.error("Got JDOMException", e);
			} catch (IOException e1) {
				logger.error("Got IOException", e);
			}
			this.openSaveFile = file;
			this.tdl = null;
			this.tdl = new TodoList("ToDoList");
			this.tdl.setXML(xml);
			this.refreshJTree();
		}
	}

	/**
	 * This should refresh the Tree component
	 *
	 */
	private void refreshJTree() {
		int size = this.getJSplitPane().getDividerLocation();
		this.getJSplitPane().remove(this.getJScrollPane());
		this.jScrollPane = null;
		this.jTree = null;
		this.getJSplitPane().setLeftComponent(this.getJScrollPane());
		this.getJSplitPane().setDividerLocation(size);
	}

	/**
	 * tries to change the content-type-encoding of an illegal xml file
	 * @param file
	 * @return
	 */
	private File sanitizeFile(File file) {
		logger.info("Sanitizing File");
		FileReader fr = null;
		BufferedReader br = null;
		FileWriter fw = null;
		BufferedWriter bw = null;
		try {
			fr = new FileReader(file);
			br = new BufferedReader(fr);
			File tempFile = new File(XMLTools.getTempFilename());
			fw = new FileWriter(tempFile);
			bw = new BufferedWriter(fw);
			String buffer = br.readLine(); // read the first line, should contain <?xml ...
			if (buffer.startsWith("<?xml")) {
				bw.write("<?xml version=\"1.0\" encoding=\"ISO8859-1\" ?>");
				bw.newLine();
			} else {
				bw.write(buffer);
				bw.newLine();
			}
			while ((buffer = br.readLine()) != null) {
				bw.write(buffer);
				logger.trace("sanitizeFile() - Writing: " + buffer);
				bw.newLine();
			}
			tempFile.deleteOnExit();
			return tempFile;
		} catch (FileNotFoundException e) {
			logger.error("FileNotFoundException", e);
		} catch (IOException e) {
			logger.error("IOException", e);
		} finally {

			try {
				br.close();
				fr.close();
				bw.close();
				fw.close();
			} catch (IOException e) {
				logger.error("IOException", e);
			}
		}
		return null;
	}

	private void saveCurrentTask() {
		if (currentTask == null)
			return;
		this.currentTask.setTitle(getTitleTextField().getText());
		this.currentTask.setPriority(getPrioritySlider().getValue());
		this.currentTask.setPercentdone(getPercentDoneSlider().getValue());
		this.currentTask.setRisk(getRiskSlider().getValue());
		this.currentTask.setStartDate(getStartDateChooser().getDate());
		this.currentTask.setDoneDate(getDoneDateChooser().getDate());
		this.currentTask.setDueDate(getDueDateChooser().getDate());
		this.currentTask.setStatus(getStatusTextField().getText());
		this.currentTask.setAllocatedBy(getAllocatedByTextField().getText());
		this.currentTask.setPerson(getPersonTextField().getText());
		this.currentTask.setDepends(getDependsTextField().getText());
		this.currentTask.setCreationDate(getCreationDateChooser().getDate());
		this.currentTask.setTimeSpent(Double.parseDouble(getTimeSpentTextField().getText()));
		this.currentTask.setTimeSpentUnits(getTimeSpentUnitsTextField().getText());
		this.currentTask.setExternalId(getExternalIdTextField().getText());
		this.currentTask.setTimeEstimated(Double.parseDouble(getTimeEstTextField().getText()));
		this.currentTask.setTimeEstimatedUnits(getTimeEstUnitsTextField().getText());
		this.currentTask.setCategory(getCategoryTextField().getText());
		this.currentTask.setCost(Double.parseDouble(getCostTextField().getText()));
		this.currentTask.setComments(getCommentsTextArea().getText());
	}

	private void saveToFile(File file) {
		IExport export = new XMLExport();
		export.setOutputFile(file);
		export.setTodoList(this.tdl);
		export.export();
	}

	private void showJList(Comparator<ITask> comp) {
		currentTaskList = null;
		currentTaskList = this.tdl.getAsArray();
		Arrays.sort(currentTaskList, comp);
		int size = this.getJSplitPane().getDividerLocation();
		this.getJSplitPane().remove(this.getJScrollPane());
		this.getJSplitPane().remove(this.getJScrollPaneList());
		this.jScrollPane = null;
		this.jTree = null;
		this.jList = null;
		for (int i = 0; i < currentTaskList.length; i++) {
			logger.info(currentTaskList[i].getTitle() + " P:" + currentTaskList[i].getPriority() + " R:"
					+ currentTaskList[i].getRisk() + " DD:" + currentTaskList[i].getDueDate());
		}
		this.getJSplitPane().setLeftComponent(this.getJScrollPaneList());
		this.getJSplitPane().setDividerLocation(size);
	}

}
