package core.gui.views;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import org.apache.log4j.Logger;

import core.structures.ITask;
import core.structures.TodoList;

public class ListView implements IView {

	static Logger				logger			= Logger.getLogger(ListView.class);

	private Comparator<ITask>	comp			= null;

	private JSplitPane			jSplitPane		= null;

	private JScrollPane			jScrollPane		= null;

	private JPanel				jTaskDetailPane	= null;

	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			logger.trace("getJScrollPane() - Generating new Scroll Pane");
			jScrollPane = new JScrollPane(getJList());
		}
		return jScrollPane;
	}

	private JList	jList	= null;

	private JList getJList() {
		if (jList == null) {
			jList = new JList(this.currentTaskList);
			jList.addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent e) {
					int index = ListView.this.jList.locationToIndex(e.getPoint());
					logger.trace("Clicked on Item " + index);
					ListView.this.showTask(ListView.this.currentTaskList[index]);
				}
				public void mouseEntered(MouseEvent e) {
					// not necessary
				}
				public void mouseExited(MouseEvent e) {
					// not necessary
				}
				public void mousePressed(MouseEvent e) {
					// not necessary
				}
				public void mouseReleased(MouseEvent e) {
					// not necessary
				}
			});
		}
		return jList;
	}

	private JPanel getTaskDetailsPane() {
		if (jTaskDetailPane == null) {
			jTaskDetailPane = new JPanel();
		}
		return jTaskDetailPane;
	}

	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, getJScrollPane(), getTaskDetailsPane());
			jSplitPane.setDividerLocation(200);
			jSplitPane.setDividerSize(5);
			jSplitPane.setDoubleBuffered(true);
		}
		return jSplitPane;
	}

	private ITask[]		currentTaskList	= null;

	private TodoList	todolist		= null;

	@SuppressWarnings("unchecked")
	public ListView(Comparator comp, TodoList tdl) {
		this.comp = comp;
		this.todolist = tdl;
		Arrays.sort(currentTaskList, this.comp);
	}

	public JComponent getView() {
		return getJSplitPane();
	}

	public void setTodolist(TodoList tdl) {
		this.todolist = tdl;
		this.updateView();
	}

	public void updateView() {
		currentTaskList = null;
		currentTaskList = this.todolist.getAsArray();
		Arrays.sort(currentTaskList, comp);
		int size = this.getJSplitPane().getDividerLocation();
		this.getJSplitPane().remove(this.getJScrollPane());
		this.jScrollPane = null;
		this.jList = null;
		for (int i = 0; i < currentTaskList.length; i++) {
			logger.info(currentTaskList[i].getTitle() + " P:" + currentTaskList[i].getPriority() + " R:"
					+ currentTaskList[i].getRisk() + " DD:" + currentTaskList[i].getDueDate());
		}
		this.getJSplitPane().setLeftComponent(this.getJScrollPane());
		this.getJSplitPane().setDividerLocation(size);
	}

	public void showTask(ITask task) {
		// TODO Auto-generated method stub

	}

}
