package core.gui.views.treemap;

import java.awt.Color;
import java.awt.Paint;

import org.apache.log4j.Logger;

import treemap.TMComputeDraw;
import treemap.TMExceptionBadTMNodeKind;
import treemap.TMNode;
import treemap.TMNodeAdapter;
import core.structures.ITask;
import core.structures.TodoList;

public class PriorityDraw implements TMComputeDraw {

	static Logger	logger	= Logger.getLogger(PriorityDraw.class);

	public Paint getFilling(TMNodeAdapter arg0) throws TMExceptionBadTMNodeKind {
		try {
			if (((ITask) arg0.getNode()).getRisk() > 8)
				return Color.red;
			if (((ITask) arg0.getNode()).getRisk() > 6)
				return Color.orange;
			if (((ITask) arg0.getNode()).getRisk() > 4)
				return Color.yellow;
			return Color.green;
		} catch (ClassCastException e) {
			return Color.gray;
		}
	}

	public String getTitle(TMNodeAdapter arg0) throws TMExceptionBadTMNodeKind {
		TMNode node = arg0.getNode();
		if (node instanceof ITask) {
			ITask tNode = (ITask) node;
			logger.trace("Returning Task Title: " + tNode.getTitle() + " |");
			return tNode.getTitle();
		} else if (node instanceof TodoList) {
			TodoList tNode = (TodoList) node;
			logger.trace("Returning TodoList Project Name: " + tNode.toString() + " |");
			return tNode.toString();
		} else {
			logger.trace("Throwing Exception, neither TDL nor Task");
			throw new TMExceptionBadTMNodeKind(this, node);
		}
	}

	public Paint getTitleColor(TMNodeAdapter arg0) throws TMExceptionBadTMNodeKind {
		return Color.black;
	}

	public String getTooltip(TMNodeAdapter arg0) throws TMExceptionBadTMNodeKind {
		try {
			ITask t = (ITask) arg0.getNode();
			StringBuilder sb = new StringBuilder();
			sb.append(t.getTitle());
			sb.append("\nPercent done:\n");
			sb.append(t.getPercentdone());
			sb.append("\nRisk:\n");
			sb.append(t.getRisk());
			sb.append("\nPriority:\n");
			sb.append(t.getPriority());
			return sb.toString();
		} catch (ClassCastException e) {
			return ((TodoList) arg0.getNode()).getProjectName() + " ";
		}
	}

	public boolean isCompatibleWith(TMNode arg0) {
		if (arg0 instanceof ITask || arg0 instanceof TodoList) {
			return true;
		}
		return false;
	}

}
