package core.structures;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import javax.swing.tree.DefaultMutableTreeNode;

import org.jdom.Content;

/**
 *  A Taskinterface.
 *
 *  Acording to the Dependency Inversion Principle all relationships should terminate in an interface.
 *  This is why this interface was extracted.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public interface ITask extends Collection, List {

	/**
	 * @see Object.equals(Object)
	 * FIXME This doesn't work! Check out why!
	 */
	public abstract boolean equals(Object o);

	/**
	 * @see List.add(int, Object)
	 */
	public abstract void add(int index, Object element);

	/**
	 * @see List.add(Object)
	 */
	public abstract boolean add(Object o);

	/**
	 * @see Collection.addAll(Collection)
	 */
	@SuppressWarnings("unchecked")
	public abstract boolean addAll(Collection c);

	/**
	 * @see Collection.addAll(int, Collection)
	 */
	@SuppressWarnings("unchecked")
	public abstract boolean addAll(int index, Collection c);

	/**
	 * Adds a task to the list of subtasks.
	 * @param t The Task to add to the List
	 */
	public abstract void addTask(ITask t);

	/**
	 * @see List.clear()
	 */
	public abstract void clear();

	/**
	 * @see Collection.contains(Object)
	 */
	public abstract boolean contains(Object o);

	/**
	 * @see Collection.containsAll(Collection)
	 */
	public abstract boolean containsAll(Collection c);

	/**
	 * @see List.get(int)
	 */
	public abstract Object get(int index);

	/**
	 * @return the allocatedBy
	 */
	public abstract String getAllocatedBy();

	public abstract DefaultMutableTreeNode getAsTreeNode();

	/**
	 * @return the category
	 */
	public abstract String getCategory();

	/**
	 * @return the comments
	 */
	public abstract String getComments();

	/**
	 * @return the cost
	 */
	public abstract double getCost();

	/**
	 * @return the creationDate
	 */
	public abstract Date getCreationDate();

	/**
	 * @return the depends
	 */
	public abstract String getDepends();

	/**
	 * @return the doneDate
	 */
	public abstract Date getDoneDate();

	/**
	 * @return the dueDate
	 */
	public abstract Date getDueDate();

	/**
	 * @return the externalId
	 */
	public abstract String getExternalId();

	/**
	 * @return the fileReferingPath
	 */
	public abstract String getFileReferingPath();

	/**
	 * @return the id
	 */
	public abstract int getId();

	/**
	 * @return the lastModifiedDate
	 */
	public abstract Date getLastModifiedDate();

	/**
	 * @return the numCategory
	 */
	public abstract int getNumCategory();

	/**
	 * @return the percentdone
	 */
	public abstract int getPercentdone();

	/**
	 * @return the person
	 */
	public abstract String getPerson();

	/**
	 * @return the pos
	 */
	public abstract int getPos();

	/**
	 * @return the priority
	 */
	public abstract int getPriority();

	/**
	 * @return the priorityColor
	 */
	public abstract int getPriorityColor();

	/**
	 * @return the priorityWebColor
	 */
	public abstract String getPriorityWebColor();

	/**
	 * @return the risk
	 */
	public abstract int getRisk();

	/**
	 * @return the startDate
	 */
	public abstract Date getStartDate();

	/**
	 * @return the status
	 */
	public abstract String getStatus();

	/**
	 *
	 * @param i the index of the task. must be < size()
	 * @return the task at position i
	 */
	public abstract ITask getTask(int i);

	/**
	 * @return the timeEstimated
	 */
	public abstract double getTimeEstimated();

	/**
	 * @return the timeEstimatedUnits
	 */
	public abstract String getTimeEstimatedUnits();

	/**
	 * @return the timeSpent
	 */
	public abstract double getTimeSpent();

	/**
	 * @return the timeSpentUnits
	 */
	public abstract String getTimeSpentUnits();

	/**
	 * @return the title
	 */
	public abstract String getTitle();

	public abstract Content getXML();

	public abstract int hashCode();

	public abstract int indexOf(Object o);

	public abstract boolean isEmpty();

	public abstract Iterator iterator();

	public abstract int lastIndexOf(Object o);

	public abstract ListIterator listIterator();

	public abstract ListIterator listIterator(int index);

	public abstract int numTasks();

	public abstract String prettyFormat(int indent);

	public abstract Object remove(int index);

	public abstract boolean remove(Object o);

	public abstract boolean removeTask(ITask t);

	public abstract boolean removeAll(Collection c);

	/**
	 * @see Collection.retainAll(Collection)
	 */
	public abstract boolean retainAll(Collection c);

	/**
	 * @see List.set
	 */
	public abstract Object set(int index, Object element);

	/**
	 * @param allocatedBy the allocatedBy to set
	 */
	public abstract void setAllocatedBy(String allocatedBy);

	/**
	 * @param category the category to set
	 */
	public abstract void setCategory(String category);

	/**
	 * @param comments the comments to set
	 */
	public abstract void setComments(String comments);

	/**
	 * @param cost the cost to set
	 */
	public abstract void setCost(double cost);

	/**
	 * @param creationDate the creationDate to set
	 */
	public abstract void setCreationDate(Date creationDate);

	/**
	 * @param depends the depends to set
	 */
	public abstract void setDepends(String depends);

	/**
	 * @param doneDate the doneDate to set
	 */
	public abstract void setDoneDate(Date doneDate);

	/**
	 * @param dueDate the dueDate to set
	 */
	public abstract void setDueDate(Date dueDate);

	/**
	 * @param externalId the externalId to set
	 */
	public abstract void setExternalId(String externalId);

	/**
	 * @param fileReferingPath the fileReferingPath to set
	 */
	public abstract void setFileReferingPath(String fileReferingPath);

	/**
	 * @param id the id to set
	 */
	public abstract void setId(int id);

	/**
	 * @param lastModifiedDate the lastModifiedDate to set
	 */
	public abstract void setLastModifiedDate(Date lastModifiedDate);

	/**
	 * @param numCategory the numCategory to set
	 */
	public abstract void setNumCategory(int numCategory);

	/**
	 * @param percentdone the percentdone to set
	 */
	public abstract void setPercentdone(int percentdone);

	/**
	 * @param person the person to set
	 */
	public abstract void setPerson(String person);

	/**
	 * @param pos the pos to set
	 */
	public abstract void setPos(int pos);

	/**
	 * @param priority the priority to set
	 */
	public abstract void setPriority(int priority);

	/**
	 * @param priorityColor the priorityColor to set
	 */
	public abstract void setPriorityColor(int priorityColor);

	/**
	 * @param priorityWebColor the priorityWebColor to set
	 */
	public abstract void setPriorityWebColor(String priorityWebColor);

	/**
	 * @param risk the risk to set
	 */
	public abstract void setRisk(int risk);

	/**
	 * @param startDate the startDate to set
	 */
	public abstract void setStartDate(Date startDate);

	/**
	 * @param status the status to set
	 */
	public abstract void setStatus(String status);

	/**
	 * @param timeEstimated the timeEstimated to set
	 */
	public abstract void setTimeEstimated(double timeEstimated);

	/**
	 * @param timeEstimatedUnits the timeEstimatedUnits to set
	 */
	public abstract void setTimeEstimatedUnits(String timeEstimatedUnits);

	/**
	 * @param timeSpent the timeSpent to set
	 */
	public abstract void setTimeSpent(double timeSpent);

	/**
	 * @param timeSpentUnits the timeSpentUnits to set
	 */
	public abstract void setTimeSpentUnits(String timeSpentUnits);

	/**
	 * @param title the title to set
	 */
	public abstract void setTitle(String title);

	/**
	 * The number of contained tasks
	 */
	public abstract int size();

	/**
	 * @see List.subList(int, int)
	 */
	public abstract List subList(int fromIndex, int toIndex);

	public abstract Object[] toArray();

	@SuppressWarnings("unchecked")
	public abstract Object[] toArray(Object[] a);

	/**
	 * @see Object.toString()
	 */
	public abstract String toString();

	public abstract Collection<? extends ITask> getAsList();

}
